@@ -1,5 +1,5 @@
 """
-Hom family of models with regimes. 
+Hom family of models with regimes.
 """
 
 __author__ = "Luc Anselin luc.anselin@asu.edu, Pedro V. Amaral pedro.amaral@asu.edu, Daniel Arribas-Bel darribas@asu.edu"
@@ -30,8 +30,8 @@ from platform import system
 import pandas as pd
 from .output import output, _summary_iteration, _spat_pseudo_r2
 
-class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
+class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
     """
     GMM method for a spatial error model with homoskedasticity, with regimes,
     results and diagnostics; based on Drukker et al. (2013) :cite:`Drukker2013`, following
@@ -39,12 +39,12 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -336,16 +336,16 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         latex=False,
         hard_bound=False,
     ):
-
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
         self.constant_regi = constant_regi
         self.cols2regi = cols2regi
         self.regime_err_sep = regime_err_sep
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_w = USER.set_name_w(name_w, w)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.n = n
         self.y = y
@@ -354,7 +354,7 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
 
-        if slx_lags >0:
+        if slx_lags > 0:
             lag_x = get_lags(w, x_constant, slx_lags)
             x_constant = np.hstack((x_constant, lag_x))
             name_x += USER.set_name_spatial_lags(name_x, slx_lags)
@@ -450,23 +450,36 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             if slx_lags == 0:
                 self.title = "GM SPATIALLY WEIGHTED MODEL (HOM) - REGIMES"
             else:
-                self.title = "GM SPATIALLY WEIGHTED MODEL + SLX (Error-HOM) - REGIMES"          
+                self.title = "GM SPATIALLY WEIGHTED MODEL + SLX (Error-HOM) - REGIMES"
             self.name_x.append("lambda")
             self.kf += 1
             self.chow = REGI.Chow(self)
             self._cache = {}
             self.A1 = A1
-            self.output = pd.DataFrame(self.name_x,
-                                       columns=['var_names'])
-            self.output['var_type'] = ['x']*(len(self.name_x)-1)+['lambda']
-            self.output['regime'] = x_rlist + ['_Global']
-            self.output['equation'] = 0
+            self.output = pd.DataFrame(self.name_x, columns=["var_names"])
+            self.output["var_type"] = ["x"] * (len(self.name_x) - 1) + ["lambda"]
+            self.output["regime"] = x_rlist + ["_Global"]
+            self.output["equation"] = 0
             self.other_top = _summary_iteration(self)
             output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
     def _error_hom_regimes_multi(
-        self, y, x, regimes, w, slx_lags, cores, max_iter, epsilon, A1, cols2regi, vm, name_x, latex, hard_bound):
-
+        self,
+        y,
+        x,
+        regimes,
+        w,
+        slx_lags,
+        cores,
+        max_iter,
+        epsilon,
+        A1,
+        cols2regi,
+        vm,
+        name_x,
+        latex,
+        hard_bound,
+    ):
         regi_ids = dict(
             (r, list(np.where(np.array(regimes) == r)[0])) for r in self.regimes_set
         )
@@ -548,7 +561,9 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
         results = {}
         counter = 0
-        self.output = pd.DataFrame(columns=['var_names', 'var_type', 'regime', 'equation'])
+        self.output = pd.DataFrame(
+            columns=["var_names", "var_type", "regime", "equation"]
+        )
         for r in self.regimes_set:
             """
             if is_win:
@@ -565,26 +580,31 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                 (counter * self.kr) : ((counter + 1) * self.kr),
                 (counter * self.kr) : ((counter + 1) * self.kr),
             ] = results[r].vm
-            self.betas[
-                (counter * self.kr) : ((counter + 1) * self.kr),
-            ] = results[r].betas
-            self.u[
-                regi_ids[r],
-            ] = results[r].u
-            self.predy[
-                regi_ids[r],
-            ] = results[r].predy
-            self.e_filtered[
-                regi_ids[r],
-            ] = results[r].e_filtered
+            self.betas[(counter * self.kr) : ((counter + 1) * self.kr),] = results[
+                r
+            ].betas
+            self.u[regi_ids[r],] = results[r].u
+            self.predy[regi_ids[r],] = results[r].predy
+            self.e_filtered[regi_ids[r],] = results[r].e_filtered
             self.name_y += results[r].name_y
             self.name_x += results[r].name_x
             results[r].A1 = A1
             results[r].other_top = _summary_iteration(results[r])
-            self.output = pd.concat([self.output, pd.DataFrame({'var_names': results[r].name_x,
-                                                                'var_type': ['x'] * (len(results[r].name_x) - 1) +
-                                                                            ['lambda'],
-                                                                'regime': r, 'equation': r})], ignore_index=True)
+            self.output = pd.concat(
+                [
+                    self.output,
+                    pd.DataFrame(
+                        {
+                            "var_names": results[r].name_x,
+                            "var_type": ["x"] * (len(results[r].name_x) - 1)
+                            + ["lambda"],
+                            "regime": r,
+                            "equation": r,
+                        }
+                    ),
+                ],
+                ignore_index=True,
+            )
             counter += 1
         self.chow = REGI.Chow(self)
         self.multi = results
@@ -592,7 +612,6 @@ class GM_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
 
 class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
-
     """
     GMM method for a spatial error model with homoskedasticity, regimes and
     endogenous variables.
@@ -601,19 +620,19 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    yend         : array
+    yend         : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    endogenous variable
-    q            : array
+    q            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    external exogenous variable to use as instruments (note:
                    this should not contain any variables from x)
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -953,13 +972,14 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         latex=False,
         hard_bound=False,
     ):
-
         n = USER.check_arrays(y, x, yend, q)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
         self.constant_regi = constant_regi
         self.cols2regi = cols2regi
         self.name_ds = USER.set_name_ds(name_ds)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.name_w = USER.set_name_w(name_w, w)
         self.n = n
@@ -1106,17 +1126,20 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self.chow = REGI.Chow(self)
             self._cache = {}
             self.A1 = A1
-            self.output = pd.DataFrame(self.name_z,
-                                       columns=['var_names'])
-            self.output['var_type'] = ['x'] * len(self.name_x) + ['yend'] * len(self.name_yend) + ['lambda']
-            self.output['regime'] = x_rlist + yend_rlist + ['_Global']
-            self.output['equation'] = 0
+            self.output = pd.DataFrame(self.name_z, columns=["var_names"])
+            self.output["var_type"] = (
+                ["x"] * len(self.name_x) + ["yend"] * len(self.name_yend) + ["lambda"]
+            )
+            self.output["regime"] = x_rlist + yend_rlist + ["_Global"]
+            self.output["equation"] = 0
             if summ:
                 self.other_top = _summary_iteration(self)
                 if slx_lags == 0:
-                    self.title = ("GM SPATIALLY WEIGHTED 2SLS (HOM) - REGIMES")
+                    self.title = "GM SPATIALLY WEIGHTED 2SLS (HOM) - REGIMES"
                 else:
-                    self.title = ("GM SPATIALLY WEIGHTED 2SLS WITH SLX (Error-HOM) - REGIMES")
+                    self.title = (
+                        "GM SPATIALLY WEIGHTED 2SLS WITH SLX (Error-HOM) - REGIMES"
+                    )
                 output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
     def _endog_error_hom_regimes_multi(
@@ -1141,7 +1164,6 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
         latex,
         hard_bound,
     ):
-
         regi_ids = dict(
             (r, list(np.where(np.array(regimes) == r)[0])) for r in self.regimes_set
         )
@@ -1244,7 +1266,9 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self.name_h,
         ) = ([], [], [], [], [], [])
         counter = 0
-        self.output = pd.DataFrame(columns=['var_names', 'var_type', 'regime', 'equation'])
+        self.output = pd.DataFrame(
+            columns=["var_names", "var_type", "regime", "equation"]
+        )
         for r in self.regimes_set:
             """
             if is_win:
@@ -1261,18 +1285,12 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
                 (counter * self.kr) : ((counter + 1) * self.kr),
                 (counter * self.kr) : ((counter + 1) * self.kr),
             ] = results[r].vm
-            self.betas[
-                (counter * self.kr) : ((counter + 1) * self.kr),
-            ] = results[r].betas
-            self.u[
-                regi_ids[r],
-            ] = results[r].u
-            self.predy[
-                regi_ids[r],
-            ] = results[r].predy
-            self.e_filtered[
-                regi_ids[r],
-            ] = results[r].e_filtered
+            self.betas[(counter * self.kr) : ((counter + 1) * self.kr),] = results[
+                r
+            ].betas
+            self.u[regi_ids[r],] = results[r].u
+            self.predy[regi_ids[r],] = results[r].predy
+            self.e_filtered[regi_ids[r],] = results[r].e_filtered
             self.name_y += results[r].name_y
             self.name_x += results[r].name_x
             self.name_yend += results[r].name_yend
@@ -1280,22 +1298,37 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
             self.name_z += results[r].name_z
             self.name_h += results[r].name_h
             if add_lag != False:
-                self.predy_e[
-                    regi_ids[r],
-                ] = results[r].predy_e
-                self.e_pred[
-                    regi_ids[r],
-                ] = results[r].e_pred
+                self.predy_e[regi_ids[r],] = results[r].predy_e
+                self.e_pred[regi_ids[r],] = results[r].e_pred
                 results[r].other_top = _spat_pseudo_r2(results[r])
-                v_type = ['x'] * len(results[r].name_x) + ['yend'] * (len(results[r].name_yend)-1) + ['rho','lambda']
+                v_type = (
+                    ["x"] * len(results[r].name_x)
+                    + ["yend"] * (len(results[r].name_yend) - 1)
+                    + ["rho", "lambda"]
+                )
             else:
                 results[r].other_top = ""
-                v_type = ['x'] * len(results[r].name_x) + ['yend'] * len(results[r].name_yend) + ['lambda']
+                v_type = (
+                    ["x"] * len(results[r].name_x)
+                    + ["yend"] * len(results[r].name_yend)
+                    + ["lambda"]
+                )
             results[r].A1 = A1
             results[r].other_top += _summary_iteration(results[r])
-            self.output = pd.concat([self.output, pd.DataFrame({'var_names': results[r].name_z,
-                                                                'var_type': v_type,
-                                                                'regime': r, 'equation': r})], ignore_index=True)
+            self.output = pd.concat(
+                [
+                    self.output,
+                    pd.DataFrame(
+                        {
+                            "var_names": results[r].name_z,
+                            "var_type": v_type,
+                            "regime": r,
+                            "equation": r,
+                        }
+                    ),
+                ],
+                ignore_index=True,
+            )
             counter += 1
         self.chow = REGI.Chow(self)
         self.multi = results
@@ -1303,7 +1336,6 @@ class GM_Endog_Error_Hom_Regimes(RegressionPropsY, REGI.Regimes_Frame):
 
 
 class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
-
     """
     GMM method for a spatial lag and error model with homoskedasticity,
     regimes and endogenous variables, with results and diagnostics;
@@ -1312,19 +1344,19 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    yend         : array
+    yend         : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    endogenous variable
-    q            : array
+    q            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    external exogenous variable to use as instruments (note:
                    this should not contain any variables from x)
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -1351,7 +1383,7 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
                    the spatial parameter is fixed across regimes.
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
-                   If slx_lags>0, the specification becomes of the GNSM type.     
+                   If slx_lags>0, the specification becomes of the GNSM type.
     w_lags       : integer
                    Orders of W to include as instruments for the spatially
                    lagged dependent variable. For example, w_lags=1, then
@@ -1706,16 +1738,16 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
         latex=False,
         hard_bound=False,
     ):
-
         if regime_lag_sep and not regime_err_sep:
-            set_warn(self, "regime_err_sep set to True when regime_lag_sep=True.")                
+            set_warn(self, "regime_err_sep set to True when regime_lag_sep=True.")
             regime_err_sep = True
         if regime_err_sep and not regime_lag_sep:
-            set_warn(self, "regime_err_sep set to False when regime_lag_sep=False.")                
+            set_warn(self, "regime_err_sep set to False when regime_lag_sep=False.")
             regime_err_sep = False
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
         x_constant, name_x, warn = USER.check_constant(x, name_x, just_rem=True)
         set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
@@ -1723,23 +1755,39 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
         self.name_y = USER.set_name_y(name_y)
         name_yend = USER.set_name_yend(name_yend, yend)
         name_q = USER.set_name_q(name_q, q)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
 
         if regime_err_sep and any(col != True for col in cols2regi):
-            set_warn(self, "All coefficients must vary across regimes if regime_err_sep = True, so setting cols2regi = 'all'.")
+            set_warn(
+                self,
+                "All coefficients must vary across regimes if regime_err_sep = True, so setting cols2regi = 'all'.",
+            )
             cols2regi = "all"
 
         if slx_lags > 0:
-            yend2, q2, wx = set_endog(y, x_constant, w, yend, q, w_lags, lag_q, slx_lags)
+            yend2, q2, wx = set_endog(
+                y, x_constant, w, yend, q, w_lags, lag_q, slx_lags
+            )
             x_constant = np.hstack((x_constant, wx))
             name_slx = USER.set_name_spatial_lags(name_x, slx_lags)
-            name_q.extend(USER.set_name_q_sp(name_slx[-len(name_x):], w_lags, name_q, lag_q, force_all=True))
-            name_x += name_slx   
-            cols2regi = REGI.check_cols2regi(constant_regi, cols2regi, x_constant[:, :-1], yend=yend2, add_cons=False)
+            name_q.extend(
+                USER.set_name_q_sp(
+                    name_slx[-len(name_x) :], w_lags, name_q, lag_q, force_all=True
+                )
+            )
+            name_x += name_slx
+            cols2regi = REGI.check_cols2regi(
+                constant_regi, cols2regi, x_constant[:, :-1], yend=yend2, add_cons=False
+            )
         else:
-            name_q.extend(USER.set_name_q_sp(name_x, w_lags, name_q, lag_q, force_all=True))
+            name_q.extend(
+                USER.set_name_q_sp(name_x, w_lags, name_q, lag_q, force_all=True)
+            )
             yend2, q2 = yend, q
-            cols2regi = REGI.check_cols2regi(constant_regi, cols2regi, x_constant, yend=yend2, add_cons=False)
-            
+            cols2regi = REGI.check_cols2regi(
+                constant_regi, cols2regi, x_constant, yend=yend2, add_cons=False
+            )
+
         self.regimes_set = REGI._get_regimes_set(regimes)
         self.regimes = regimes
         USER.check_regimes(self.regimes_set, n, x_constant.shape[1])
@@ -1758,7 +1806,6 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
             if slx_lags == 0:
                 yend2, q2 = set_endog(y, x_constant, w, yend2, q2, w_lags, lag_q)
 
-
         name_yend.append(USER.set_name_yend_sp(self.name_y))
 
         GM_Endog_Error_Hom_Regimes.__init__(
@@ -1800,8 +1847,10 @@ class GM_Combo_Hom_Regimes(GM_Endog_Error_Hom_Regimes):
             if slx_lags == 0:
                 self.title = "GM SPATIALLY WEIGHTED 2SLS-COMBO MODEL (HOM) - REGIMES"
             else:
-                self.title = "GM SPATIALLY WEIGHTED 2SLS-COMBO WITH SLX (GNSM-HOM) - REGIMES"
-            self.output.iat[-2, self.output.columns.get_loc('var_type')] = 'rho'
+                self.title = (
+                    "GM SPATIALLY WEIGHTED 2SLS-COMBO WITH SLX (GNSM-HOM) - REGIMES"
+                )
+            self.output.iat[-2, self.output.columns.get_loc("var_type")] = "rho"
             self.other_top = _spat_pseudo_r2(self)
             self.other_top += _summary_iteration(self)
             output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
@@ -1828,14 +1877,20 @@ def _work_error(
     y_r = y[regi_ids[r]]
     x_r = x[regi_ids[r]]
     model = BaseGM_Error_Hom(
-        y_r, x_r, w_r.sparse, max_iter=max_iter, epsilon=epsilon, A1=A1, hard_bound=hard_bound,
+        y_r,
+        x_r,
+        w_r.sparse,
+        max_iter=max_iter,
+        epsilon=epsilon,
+        A1=A1,
+        hard_bound=hard_bound,
     )
     set_warn(model, warn)
     model.w = w_r
     if slx_lags == 0:
         model.title = "GM SPATIALLY WEIGHTED 2SLS (HOM) - REGIME %s" % r
     else:
-        model.title = "GM SPATIALLY WEIGHTED 2SLS + SLX (Error-HOM) - REGIME %s" % r    
+        model.title = "GM SPATIALLY WEIGHTED 2SLS + SLX (Error-HOM) - REGIME %s" % r
     model.name_ds = name_ds
     model.name_y = "%s_%s" % (str(r), name_y)
     model.name_x = ["%s_%s" % (str(r), i) for i in name_x]
@@ -1880,7 +1935,15 @@ def _work_endog_error(
         )
     x_constant = USER.check_constant(x_r)
     model = BaseGM_Endog_Error_Hom(
-        y_r, x_r, yend_r, q_r, w_r.sparse, max_iter=max_iter, epsilon=epsilon, A1=A1, hard_bound=hard_bound,
+        y_r,
+        x_r,
+        yend_r,
+        q_r,
+        w_r.sparse,
+        max_iter=max_iter,
+        epsilon=epsilon,
+        A1=A1,
+        hard_bound=hard_bound,
     )
     set_warn(model, warn)
     if add_lag != False:
@@ -1891,19 +1954,19 @@ def _work_endog_error(
         set_warn(model, warn)
     if slx_lags == 0:
         if add_lag != False:
-            model.title = "GM SPATIALLY WEIGHTED 2SLS-COMBO MODEL (HOM)- REGIME %s" % r            
+            model.title = "GM SPATIALLY WEIGHTED 2SLS-COMBO MODEL (HOM)- REGIME %s" % r
         else:
             model.title = "GM SPATIALLY WEIGHTED 2SLS (HOM) - REGIME %s" % r
     else:
         if add_lag != False:
-            model.title = "GM SPATIAL COMBO MODEL + SLX (GNSM-HOM) - REGIME %s" % r   
+            model.title = "GM SPATIAL COMBO MODEL + SLX (GNSM-HOM) - REGIME %s" % r
         else:
             model.title = "GM SPATIALLY WEIGHTED 2SLS + SLX (Error-HOM) - REGIME %s" % r
     model.name_ds = name_ds
     model.name_y = "%s_%s" % (str(r), name_y)
     model.name_x = ["%s_%s" % (str(r), i) for i in name_x]
     model.name_yend = ["%s_%s" % (str(r), i) for i in name_yend]
-    model.name_z = model.name_x + model.name_yend + [str(r)+"_lambda"]
+    model.name_z = model.name_x + model.name_yend + [str(r) + "_lambda"]
     model.name_q = ["%s_%s" % (str(r), i) for i in name_q]
     model.name_h = model.name_x + model.name_q
     model.name_w = name_w
@@ -1926,9 +1989,9 @@ if __name__ == "__main__":
     import numpy as np
     import libpysal
 
-    db = libpysal.io.open(libpysal.examples.get_path('columbus.dbf'),'r')
+    db = libpysal.io.open(libpysal.examples.get_path("columbus.dbf"), "r")
     y = np.array(db.by_col("HOVAL"))
-    y = np.reshape(y, (49,1))
+    y = np.reshape(y, (49, 1))
     X = []
     X.append(db.by_col("INC"))
     X = np.array(X).T
@@ -1939,16 +2002,30 @@ if __name__ == "__main__":
     q.append(db.by_col("DISCBD"))
     q = np.array(q).T
 
-    r_var = 'NSA'
+    r_var = "NSA"
     regimes = db.by_col(r_var)
 
     w = libpysal.weights.Rook.from_shapefile(libpysal.examples.get_path("columbus.shp"))
-    w.transform = 'r'
-    #reg = GM_Error_Hom_Regimes(y, X, regimes, w=w, name_x=['inc'], name_y='hoval', name_ds='columbus', vm=True,
+    w.transform = "r"
+    # reg = GM_Error_Hom_Regimes(y, X, regimes, w=w, name_x=['inc'], name_y='hoval', name_ds='columbus', vm=True,
     #                           regime_err_sep=True)
-    #reg = GM_Endog_Error_Hom_Regimes(y, X, yd, q, regimes, w=w, name_x=['inc'], name_y='hoval', name_yend=['crime'],
+    # reg = GM_Endog_Error_Hom_Regimes(y, X, yd, q, regimes, w=w, name_x=['inc'], name_y='hoval', name_yend=['crime'],
     #                         name_q=['discbd'], name_ds='columbus',vm=True, regime_err_sep=True)
-    reg = GM_Combo_Hom_Regimes(y, X, regimes, yd, q, w=w, name_x=['inc'], name_y='hoval', name_yend=['crime'],
-                       name_q=['discbd'], name_ds='columbus', vm=True, regime_err_sep=False, regime_lag_sep=False)
+    reg = GM_Combo_Hom_Regimes(
+        y,
+        X,
+        regimes,
+        yd,
+        q,
+        w=w,
+        name_x=["inc"],
+        name_y="hoval",
+        name_yend=["crime"],
+        name_q=["discbd"],
+        name_ds="columbus",
+        vm=True,
+        regime_err_sep=False,
+        regime_lag_sep=False,
+    )
     print(reg.output)
-    print(reg.summary)
\ No newline at end of file
+    print(reg.summary)
