@@ -21,12 +21,12 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     w            : pysal W object
@@ -350,9 +350,9 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
     ------------------------------------------------------------------------------------
                 Variable     Coefficient       Std.Error     t-Statistic     Probability
     ------------------------------------------------------------------------------------
-              0_CONSTANT       0.3964290       0.2481634       1.5974512       0.1103544
-                  0_PS90       0.6558330       0.0966268       6.7872800       0.0000000
-                  0_UE90       0.4870394       0.0362863      13.4221336       0.0000000
+              0_CONSTANT         0.39643         0.24816         1.59745         0.11035
+                  0_PS90         0.65583         0.09663         6.78728         0.00000
+                  0_UE90         0.48704         0.03629        13.42213         0.00000
     ------------------------------------------------------------------------------------
     Regimes variable: SOUTH
     <BLANKLINE>
@@ -369,20 +369,21 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
     ------------------------------------------------------------------------------------
                 Variable     Coefficient       Std.Error     t-Statistic     Probability
     ------------------------------------------------------------------------------------
-              1_CONSTANT       5.5983500       0.4689456      11.9381640       0.0000000
-                  1_PS90       1.1621045       0.2166740       5.3633790       0.0000001
-                  1_UE90       0.5316389       0.0594565       8.9416422       0.0000000
+              1_CONSTANT         5.59835         0.46895        11.93816         0.00000
+                  1_PS90         1.16210         0.21667         5.36338         0.00000
+                  1_UE90         0.53164         0.05946         8.94164         0.00000
     ------------------------------------------------------------------------------------
     Regimes variable: SOUTH
     ------------------------------------------------------------------------------------
+    <BLANKLINE>
     GLOBAL DIAGNOSTICS
     <BLANKLINE>
     REGIMES DIAGNOSTICS - CHOW TEST
                      VARIABLE        DF        VALUE           PROB
-                     CONSTANT         1          96.129           0.0000
-                         PS90         1           4.554           0.0328
-                         UE90         1           0.410           0.5220
-                  Global test         3         680.960           0.0000
+                CONSTANT              1         96.129           0.0000
+                    PS90              1          4.554           0.0328
+                    UE90              1          0.410           0.5220
+             Global test              3        680.960           0.0000
     ================================ END OF REPORT =====================================
     """
 
@@ -415,7 +416,7 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
     ):
 
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
+        y, name_y = USER.check_y(y, n, name_y)
         USER.check_robust(robust, gwk)
         if robust == "hac":
             if regime_err_sep:
@@ -424,28 +425,26 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
                     "Error by regimes is not available for HAC estimation. The error by regimes has been disabled for this model.",
                 )
                 regime_err_sep = False
-            if spat_diag:
-                set_warn(
-                    self,
-                    "Spatial diagnostics are not available for HAC estimation. The spatial diagnostics have been disabled for this model.",
-                )
-                spat_diag = False
+        spat_diag, warn = USER.check_spat_diag(spat_diag=spat_diag, w=w, robust=robust, slx_lags=slx_lags)
+        set_warn(self, warn)
         if robust in ["hac", "white"] and white_test:
             set_warn(
                 self,
                 "White test not available when standard errors are estimated by HAC or White correction.",
             )
             white_test = False
-        USER.check_spat_diag(spat_diag, w)
+
         x_constant, name_x, warn = USER.check_constant(x, name_x, just_rem=True)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
+        if spat_diag or moran:
+            w = USER.check_weights(w, y, slx_lags=slx_lags, w_required=True, allow_wk=True)
+        else:
+            w = USER.check_weights(w, y, slx_lags=slx_lags, allow_wk=True)
         if slx_lags > 0:
-            USER.check_weights(w, y, w_required=True)
             lag_x = get_lags(w, x_constant, slx_lags)
             x_constant = np.hstack((x_constant, lag_x))
             name_x += USER.set_name_spatial_lags(name_x, slx_lags)
-        else:
-            USER.check_weights(w, y, w_required=False)
+
         set_warn(self, warn)
         self.slx_lags = slx_lags
         self.name_x_r = USER.set_name_x(name_x, x_constant)
@@ -455,6 +454,8 @@ class OLS_Regimes(BaseOLS, REGI.Regimes_Frame, RegressionPropsY):
         self.name_gwk = USER.set_name_w(name_gwk, gwk)
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
+
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.n = n
         cols2regi = REGI.check_cols2regi(
@@ -708,30 +709,355 @@ def _work(
 
 
 class OLS_Endog_Regimes(OLS_Regimes):
+    """
+    Ordinary least squares with endogenous regimes. Based on the function skater_reg as shown in :cite:`Anselin2021`.
+
+    Parameters
+    ----------
+    y            : numpy.ndarray or pandas.Series
+                   nx1 array for dependent variable
+    x            : numpy.ndarray or pandas object
+                   Two dimensional array with n rows and one column for each
+                   independent (exogenous) variable, excluding the constant
+    w            : pysal W object
+                   Spatial weights object (required if running spatial
+                   diagnostics)
+    n_clusters   : int
+                   Number of clusters to be used in the endogenous regimes.
+                   If None (default), the number of clusters will be chosen
+                   according to the function utils.optim_k using a method 
+                   adapted from Mojena (1977)'s Rule Two
+    quorum       : int
+                   Minimum number of observations in a cluster to be considered
+                   Must be at least larger than the number of variables in x
+                   Default value is 30 or 10*k, whichever is larger.
+    trace        : boolean
+                   Sets  whether to store intermediate results of the clustering
+                   Hard-coded to True if n_clusters is None
+    name_y       : string
+                   Name of dependent variable for use in output
+    name_x       : list of strings
+                   Names of independent variables for use in output
+    name_w       : string
+                   Name of weights matrix for use in output
+    name_ds      : string
+                   Name of dataset for use in output
+    latex        : boolean
+                   Specifies if summary is to be printed in latex format
+    **kwargs     : additional keyword arguments depending on the specific model
+
+    Attributes
+    ----------
+    output       : dataframe
+                   regression results pandas dataframe
+    summary      : string
+                   Summary of regression results and diagnostics (note: use in
+                   conjunction with the print command)
+    betas        : array
+                   kx1 array of estimated coefficients
+    u            : array
+                   nx1 array of residuals
+    predy        : array
+                   nx1 array of predicted y values
+    n            : integer
+                   Number of observations
+    k            : integer
+                   Number of variables for which coefficients are estimated
+                   (including the constant)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    y            : array
+                   nx1 array for dependent variable
+    x            : array
+                   Two dimensional array with n rows and one column for each
+                   independent (exogenous) variable, including the constant
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    robust       : string
+                   Adjustment for robust standard errors
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    mean_y       : float
+                   Mean of dependent variable
+    std_y        : float
+                   Standard deviation of dependent variable
+    vm           : array
+                   Variance covariance matrix (kxk)
+    r2           : float
+                   R squared
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    ar2          : float
+                   Adjusted R squared
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    utu          : float
+                   Sum of squared residuals
+    sig2         : float
+                   Sigma squared used in computations
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    sig2ML       : float
+                   Sigma squared (maximum likelihood)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    f_stat       : tuple
+                   Statistic (float), p-value (float)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    logll        : float
+                   Log likelihood
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    aic          : float
+                   Akaike information criterion
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    schwarz      : float
+                   Schwarz information criterion
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    std_err      : array
+                   1xk array of standard errors of the betas
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    t_stat       : list of tuples
+                   t statistic; each tuple contains the pair (statistic,
+                   p-value), where each is a float
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    mulColli     : float
+                   Multicollinearity condition number
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    jarque_bera  : dictionary
+                   'jb': Jarque-Bera statistic (float); 'pvalue': p-value
+                   (float); 'df': degrees of freedom (int)
+                   Only available in dictionary 'multi' when multiple regressions
+                   (see 'multi' below for details)
+    breusch_pagan : dictionary
+                    'bp': Breusch-Pagan statistic (float); 'pvalue': p-value
+                    (float); 'df': degrees of freedom (int)
+                    Only available in dictionary 'multi' when multiple regressions
+                    (see 'multi' below for details)
+    koenker_bassett: dictionary
+                     'kb': Koenker-Bassett statistic (float); 'pvalue': p-value (float);
+                     'df': degrees of freedom (int). Only available in dictionary
+                     'multi' when multiple regressions (see 'multi' below for details).
+    white         : dictionary
+                    'wh': White statistic (float); 'pvalue': p-value (float);
+                    'df': degrees of freedom (int).
+                    Only available in dictionary 'multi' when multiple regressions
+                    (see 'multi' below for details)
+    lm_error      : tuple
+                    Lagrange multiplier test for spatial error model; tuple
+                    contains the pair (statistic, p-value), where each is a
+                    float. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    lm_lag        : tuple
+                    Lagrange multiplier test for spatial lag model; tuple
+                    contains the pair (statistic, p-value), where each is a
+                    float. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    rlm_error     : tuple
+                    Robust lagrange multiplier test for spatial error model;
+                    tuple contains the pair (statistic, p-value), where each
+                    is a float. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    rlm_lag       : tuple
+                    Robust lagrange multiplier test for spatial lag model;
+                    tuple contains the pair (statistic, p-value), where each
+                    is a float. Only available in dictionary 'multi' when
+                    multiple regressions (see 'multi' below for details)
+    lm_sarma      : tuple
+                    Lagrange multiplier test for spatial SARMA model; tuple
+                    contains the pair (statistic, p-value), where each is a
+                    float. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    moran_res     : tuple
+                    Moran's I for the residuals; tuple containing the triple
+                    (Moran's I, standardized Moran's I, p-value)
+    name_y        : string
+                    Name of dependent variable for use in output
+    name_x        : list of strings
+                    Names of independent variables for use in output
+    name_w        : string
+                    Name of weights matrix for use in output
+    name_gwk      : string
+                    Name of kernel weights matrix for use in output
+    name_ds       : string
+                    Name of dataset for use in output
+    name_regimes  : string
+                    Name of regime variable for use in the output
+    title         : string
+                    Name of the regression method used.
+                    Only available in dictionary 'multi' when multiple regressions
+                    (see 'multi' below for details)
+    sig2n         : float
+                    Sigma squared (computed with n in the denominator)
+    sig2n_k       : float
+                    Sigma squared (computed with n-k in the denominator)
+    xtx           : float
+                    :math:`X'X`. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    xtxi          : float
+                    :math:`(X'X)^{-1}`. Only available in dictionary 'multi' when multiple
+                    regressions (see 'multi' below for details)
+    regimes       : list
+                    List of n values with the mapping of each observation to
+                    a regime. Assumed to be aligned with 'x'.
+    constant_regi : string
+                    Ignored if regimes=False. Constant option for regimes.
+                    Switcher controlling the constant term setup. It may take
+                    the following values:
+
+                    *  'one': a vector of ones is appended to x and held constant across regimes.
+
+                    * 'many': a vector of ones is appended to x and considered different per regime.
+    cols2regi    : list
+                   Ignored if regimes=False. Argument indicating whether each
+                   column of x should be considered as different per regime
+                   or held constant across regimes (False).
+                   If a list, k booleans indicating for each variable the
+                   option (True if one per regime, False to be held constant).
+                   If 'all', all the variables vary by regime.
+    regime_err_sep: boolean
+                    If True, a separate regression is run for each regime.
+    kr           : int
+                   Number of variables/columns to be "regimized" or subject
+                   to change by regime. These will result in one parameter
+                   estimate by regime for each variable (i.e. nr parameters per
+                   variable)
+    kf           : int
+                   Number of variables/columns to be considered fixed or
+                   global across regimes and hence only obtain one parameter
+                   estimate.
+    nr           : int
+                   Number of different regimes in the 'regimes' list.
+    multi        : dictionary
+                   Only available when multiple regressions are estimated,
+                   i.e. when regime_err_sep=True and no variable is fixed
+                   across regimes.
+                   Contains all attributes of each individual regression.
+    SSR          : list
+                   list with the total sum of squared residuals for the model 
+                   considering all regimes for each of steps of number of regimes
+                   considered, starting with the solution with 2 regimes. 
+    clusters     : int
+                   Number of clusters considered in the endogenous regimes
+    _trace       : list
+                   List of dictionaries with the clustering results for each
+                   number of clusters tested. Only available if n_clusters is
+                   None or trace=True.
+
+    Examples
+    --------
+    >>> import libpysal
+    >>> import numpy as np
+    >>> np.set_printoptions(legacy='1.25') #to avoid printing issues with numpy floats    
+    >>> import geopandas as gpd
+    >>> from spreg import OLS_Endog_Regimes
+
+    Open data on Baltimore house sales price and characteristics in Baltimore
+    from libpysal examples using geopandas.
+
+    >>> db = gpd.read_file(libpysal.examples.get_path('baltim.shp'))
+
+    We will create a weights matrix based on contiguity.
+
+    >>> w = libpysal.weights.Queen.from_dataframe(db, use_index=True)
+    >>> w.transform = "r"
+
+    For this example, we will use the 'PRICE' column as the dependent variable and
+    the 'NROOM', 'AGE', and 'SQFT' columns as independent variables.
+    At this point, we will let the model choose the number of clusters.
+
+    >>> olsr = OLS_Endog_Regimes(y=db['PRICE'], x=db[['NROOM','AGE','SQFT']], w=w, name_w="baltim_q.gal")
+    
+    The function `print(olsr.summary)` can be used to visualize the results of the regression.
+
+    Alternatively, we can check individual attributes:
+    >>> olsr.betas
+    array([[26.24209866],
+           [ 2.40329959],
+           [-0.24183707],
+           [ 0.45714794],
+           [19.84817747],
+           [ 5.12117483],
+           [-0.65466516],
+           [ 1.10034154]])
+    >>> olsr.SSR
+    [68840.74965798721, 62741.55717492997]
+    >>> olsr.clusters
+    array([0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0,
+           0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1,
+           1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0,
+           1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1], dtype=int32)
+
+    We  will now set the number of clusters to 2 and run the regression again.
+
+    >>> olsr = OLS_Endog_Regimes(y=db['PRICE'], x=db[['NROOM','AGE','SQFT']], w=w, n_clusters=2, name_w="baltim_q.gal")
+
+    The function `print(olsr.summary)` can be used to visualize the results of the regression.
+
+    Alternatively, we can check individual attributes as before:
+    >>> olsr.betas
+    array([[26.24209866],
+           [ 2.40329959],
+           [-0.24183707],
+           [ 0.45714794],
+           [19.84817747],
+           [ 5.12117483],
+           [-0.65466516],
+           [ 1.10034154]])
+    >>> olsr.SSR
+    [68840.74965798721]
+    >>> olsr.clusters
+    array([0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0,
+           0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1,
+           1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0,
+           1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0,
+           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1], dtype=int32)
+
+    """
+   
     def __init__(
-        self, y, x, w, n_clusters=None, quorum=-np.inf, trace=True, **kwargs):
+        self, y, x, w, n_clusters=None, quorum=-1, trace=True, name_y=None, name_x=None, **kwargs):
 
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
-
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True)
+        x_constant, name_x, warn = USER.check_constant(x, name_x, just_rem=True)
+        set_warn(self, warn)
         # Standardize the variables
-        x_std = (x - np.mean(x, axis=0)) / np.std(x, axis=0)
+        x_std = (x_constant - np.mean(x_constant, axis=0)) / np.std(x_constant, axis=0)
+
+        if quorum < 0:
+            quorum = np.max([(x_constant.shape[1]+1)*10, 30])
 
         if not n_clusters:
-            if quorum < 0:
-                quorum = np.max([(x.shape[1]+1)*10, 30])
-            n_clusters_opt = x.shape[0]*0.70//quorum
+            n_clusters_opt = x_constant.shape[0]*0.70//quorum
             if n_clusters_opt < 2:
                 raise ValueError(
                     "The combination of the values of `N` and `quorum` is not compatible with regimes estimation.")
-            sk_reg_results = Skater_reg().fit(n_clusters_opt, w, x_std, {'reg':BaseOLS,'y':y,'x':x}, quorum=quorum, trace=True)
+            sk_reg_results = Skater_reg().fit(n_clusters_opt, w, x_std, {'reg':BaseOLS,'y':y,'x':x_constant}, quorum=quorum, trace=True)
             n_clusters = optim_k([sk_reg_results._trace[i][1][2] for i in range(1, len(sk_reg_results._trace))])
             self.clusters = sk_reg_results._trace[n_clusters-1][0]
         else:
             try:
                 # Call the Skater_reg method based on OLS
-                sk_reg_results = Skater_reg().fit(n_clusters, w, x_std, {'reg':BaseOLS,'y':y,'x':x}, quorum=quorum, trace=trace)
+                sk_reg_results = Skater_reg().fit(n_clusters, w, x_std, {'reg':BaseOLS,'y':y,'x':x_constant}, quorum=quorum, trace=trace)
                 self.clusters = sk_reg_results.current_labels_
             except Exception as e:
                 if str(e) == "one or more input arrays have more columns than rows":
@@ -742,7 +1068,7 @@ class OLS_Endog_Regimes(OLS_Regimes):
         self._trace = sk_reg_results._trace
         self.SSR = [self._trace[i][1][2] for i in range(1, len(self._trace))]
 
-        OLS_Regimes.__init__(self, y, x, regimes=self.clusters, w=w, name_regimes='Skater_reg', **kwargs)
+        OLS_Regimes.__init__(self, y, x_constant, regimes=self.clusters, w=w, name_regimes='Skater_reg', name_y=name_y, name_x=name_x, **kwargs)
 
 def _test():
     import doctest
@@ -757,7 +1083,7 @@ if __name__ == "__main__":
     _test()
     import numpy as np
     import libpysal
-    import pysal
+    from spreg import OLS_Regimes
 
     db = libpysal.io.open(libpysal.examples.get_path("NAT.dbf"), "r")
     y_var = "HR90"
@@ -768,7 +1094,6 @@ if __name__ == "__main__":
     regimes = db.by_col(r_var)
     w = libpysal.weights.Rook.from_shapefile(libpysal.examples.get_path("NAT.shp"))
     w.transform = "r"
-    #olsr = pysal.model.spreg.OLS_Regimes(
     olsr = OLS_Regimes(
             y,
         x,
