@@ -425,7 +425,6 @@ class SUR(BaseSUR, REGI.Regimes_Frame):
         name_w=None,
         name_regimes=None,
     ):
-
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_w = USER.set_name_w(name_w, w)
         self.n_eq = len(bigy.keys())
@@ -817,7 +816,6 @@ class ThreeSLS(BaseThreeSLS, REGI.Regimes_Frame):
         name_ds=None,
         name_regimes=None,
     ):
-
         self.name_ds = USER.set_name_ds(name_ds)
         self.n_eq = len(bigy.keys())
 
@@ -996,14 +994,13 @@ if __name__ == "__main__":
     _test()
     import numpy as np
     import libpysal
-    from .sur_utils import sur_dictxy, sur_dictZ
+    from .sur_utils import sur_dictxy
     from libpysal.examples import load_example
 
     nat = load_example("Natregimes")
     db = libpysal.io.open(nat.get_path("NAT.dbf"), "r")
     y_var = ["HR80", "HR90"]
     x_var = [["PS80", "UE80"], ["PS90", "UE90"]]
-    regimes = db.by_col("SOUTH")
 
     # Example SUR
     # """
