@@ -38,7 +38,7 @@ __all__ = ["BaseSURerrorGM", "SURerrorGM", "BaseSURerrorML", "SURerrorML"]
 
 
 class BaseSURerrorGM:
-    """Base class for SUR Error estimation by Generalized Moment Estimation
+    """Base class for SUR Error estimation by Generalized Moments
 
     Parameters
     ----------
@@ -162,7 +162,7 @@ def _momentsGM_sur_Error(w, u):
 
 class SURerrorGM(BaseSURerrorGM, REGI.Regimes_Frame):
     """
-    User class for SUR Error estimation by Maximum Likelihood
+    User class for SUR Error estimation by Generalized Moments
 
     Parameters
     ----------
@@ -328,7 +328,6 @@ class SURerrorGM(BaseSURerrorGM, REGI.Regimes_Frame):
         name_w=None,
         name_regimes=None,
     ):
-
         # check on variable names for listing results
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_w = USER.set_name_w(name_w, w)
@@ -553,7 +552,7 @@ class BaseSURerrorML:
             bigE = sur_resids(self.bigy, self.bigX, b1)
             res = minimize(
                 clik,
-                lam,
+                np.array(lam).flatten(),
                 args=(self.n, self.n2, self.n_eq, bigE, I, WS),
                 method="L-BFGS-B",
                 bounds=lambdabounds,
@@ -777,7 +776,6 @@ class SURerrorML(BaseSURerrorML, REGI.Regimes_Frame):
         name_w=None,
         name_regimes=None,
     ):
-
         # need checks on match between bigy, bigX dimensions
         # check on variable names for listing results
         self.name_ds = USER.set_name_ds(name_ds)
@@ -1119,7 +1117,7 @@ if __name__ == "__main__":
         bigy0,
         bigX0,
         w,
-        regimes=regimes,
+        # regimes=regimes,
         name_bigy=bigyvars0,
         name_bigX=bigXvars0,
         name_w="natqueen",
