@@ -7,6 +7,7 @@ __author__ = "Luc Anselin lanselin@gmail.com,    \
 
 import numpy as np
 import numpy.linalg as la
+import pandas as pd
 from .utils import spdot
 
 __all__ = [
@@ -30,11 +31,11 @@ def sur_dictxy(db, y_vars, x_vars, space_id=None, time_id=None):
     Parameters
     ----------
     db          : data object
-                  created by libpysal.io.open
-    y_vars      : array
+                  created by libpysal.io.open or pandas.DataFrame
+    y_vars      : list
                   list of lists with variable name(s) for dependent var
                   (Note must be a list of lists, even in splm case)
-    x_vars      : array
+    x_vars      : list
                   list of lists with variable names for explanatory vars
     space_id    : variable with space ID
                   used for splm format
@@ -62,8 +63,8 @@ def sur_dictxy(db, y_vars, x_vars, space_id=None, time_id=None):
         bigy = {}
         bigy_vars = dict((r, y_vars[r]) for r in range(n_eq))
         bigy = dict((r, np.resize(y[:, r], (n, 1))) for r in range(n_eq))
-        if not (len(x_vars) == n_eq):  # CHANGE into exception
-            print("Error: mismatch variable lists")
+        if not (len(x_vars) == n_eq):
+            raise Exception("Error: mismatch variable lists")
         bigX = {}
         bigX_vars = {}
         for r in range(n_eq):
@@ -79,8 +80,8 @@ def sur_dictxy(db, y_vars, x_vars, space_id=None, time_id=None):
             k = litxc.shape[1]
         return (bigy, bigX, bigy_vars, bigX_vars)
     elif len(y_vars) == 1:  # splm format
-        if not (time_id):  # CHANGE into exception
-            print("Error: time id must be specified")
+        if not (time_id):
+            raise Exception("Error: time id must be specified")
         try:
             y = np.array([db[name] for name in y_vars]).T
         except:
@@ -137,7 +138,7 @@ def sur_dictxy(db, y_vars, x_vars, space_id=None, time_id=None):
             bigX_vars[r] = [i + "_" + tt3[r] for i in xvars]
         return (bigy, bigX, bigy_vars, bigX_vars)
     else:
-        print("error message, but should never be here")
+        raise Exception("error message, but should never be here")
 
 
 def sur_dictZ(db, z_vars, form="spreg", const=False, space_id=None, time_id=None):
@@ -147,7 +148,7 @@ def sur_dictZ(db, z_vars, form="spreg", const=False, space_id=None, time_id=None
     Parameters
     ----------
     db          : data object
-                  created by libpysal.io.open
+                  created by libpysal.io.open or pandas.DataFrame
     varnames    : array
                   list of lists with variable name(s)
                   (Note must be a list of lists, even in splm case)
@@ -290,7 +291,7 @@ def sur_dict2mat(dicts):
 
     """
     n_dicts = len(dicts.keys())
-    #mat = np.vstack((dicts[t] for t in range(n_dicts)))
+    # mat = np.vstack((dicts[t] for t in range(n_dicts)))
     mat = np.vstack([dicts[t] for t in range(n_dicts)])
 
     return mat
@@ -394,11 +395,11 @@ def sur_est(bigXX, bigXy, bigE, bigK):
         for t in range(n_eq):
             sxy = sxy + sigi[r, t] * bigXy[(r, t)]
         sigiXy[r] = sxy
-    #xsigy = np.vstack((sigiXy[t] for t in range(n_eq)))
+    # xsigy = np.vstack((sigiXy[t] for t in range(n_eq)))
     xsigy = np.vstack(tuple(sigiXy[t] for t in range(n_eq)))
-    #xsigx = np.vstack(((np.hstack(sigiXX[(r, t)] for t in range(n_eq))) for r in range(n_eq)))
+    # xsigx = np.vstack(((np.hstack(sigiXX[(r, t)] for t in range(n_eq))) for r in range(n_eq)))
     array_lists = [[sigiXX[(r, t)] for t in range(n_eq)] for r in range(n_eq)]
-    xsigx = np.vstack([np.hstack(arr_list) for arr_list in array_lists])    
+    xsigx = np.vstack([np.hstack(arr_list) for arr_list in array_lists])
     varb = la.inv(xsigx)
     beta = np.dot(varb, xsigy)
     bSUR = sur_mat2dict(beta, bigK)
@@ -426,7 +427,7 @@ def sur_resids(bigy, bigX, beta):
 
     """
     n_eq = len(bigy.keys())
-    #bigE = np.hstack((bigy[r] - spdot(bigX[r], beta[r])) for r in range(n_eq))
+    # bigE = np.hstack((bigy[r] - spdot(bigX[r], beta[r])) for r in range(n_eq))
     bigE = np.hstack(tuple(bigy[r] - spdot(bigX[r], beta[r]) for r in range(n_eq)))
 
     return bigE
@@ -454,7 +455,7 @@ def sur_predict(bigy, bigX, beta):
 
     """
     n_eq = len(bigy.keys())
-    #bigYP = np.hstack(spdot(bigX[r], beta[r]) for r in range(n_eq))
+    # bigYP = np.hstack(spdot(bigX[r], beta[r]) for r in range(n_eq))
     bigYP = np.hstack([spdot(bigX[r], beta[r]) for r in range(n_eq)])
 
     return bigYP
