@@ -2,7 +2,7 @@
 Tools for different procedure estimations
 """
 
-__author__ = "Luc Anselin luc.anselin@asu.edu, \
+__author__ = "Luc Anselin lanselin@gmail.com, \
         Pedro V. Amaral pedro.amaral@asu.edu, \
         David C. Folch david.folch@asu.edu, \
         Daniel Arribas-Bel darribas@asu.edu,\
@@ -13,10 +13,13 @@ from scipy import sparse as SP
 import scipy.optimize as op
 import numpy.linalg as la
 from libpysal.weights.spatial_lag import lag_spatial
+from libpysal.cg import KDTree        # new for make_wnslx
+from scipy.sparse import coo_array,csr_array    # new for make_wnslx
 from .sputils import *
 import copy
 
 
+
 class RegressionPropsY(object):
 
     """
@@ -218,7 +221,7 @@ def get_A1_het(S):
 
 
 def get_A1_hom(s, scalarKP=False):
-    """
+    r"""
     Builds A1 for the spatial error GM estimation with homoscedasticity as in
     Drukker et al. [Drukker2011]_ (p. 9).
 
@@ -257,7 +260,7 @@ def get_A1_hom(s, scalarKP=False):
 
 
 def get_A2_hom(s):
-    """
+    r"""
     Builds A2 for the spatial error GM estimation with homoscedasticity as in
     Anselin (2011) :cite:`Anselin2011`
 
@@ -322,7 +325,7 @@ def _moments2eqs(A1, s, u):
     return [G, g]
 
 
-def optim_moments(moments_in, vcX=np.array([0]), all_par=False, start=None):
+def optim_moments(moments_in, vcX=np.array([0]), all_par=False, start=None, hard_bound=False):
     """
     Optimization of moments
     ...
@@ -341,7 +344,10 @@ def optim_moments(moments_in, vcX=np.array([0]), all_par=False, start=None):
                   solution or just the 1st. Default is 1st only.
     start       : list
                   List with initial values for the optimization
-
+    hard_bound   : boolean
+                   If true, raises an exception if the estimated spatial
+                   autoregressive parameter is outside the maximum/minimum bounds.
+                   
     Returns
     -------
     x, f, d     : tuple
@@ -367,13 +373,13 @@ def optim_moments(moments_in, vcX=np.array([0]), all_par=False, start=None):
             np.array([[float(par[0]), float(par[0]) ** 2.0]]).T, moments
         )
         start = [0.0]
-        bounds = [(-1.0, 1.0)]
+        bounds = [(-0.99, 0.99)]
     if moments[0].shape[0] == 3:
         optim_par = lambda par: foptim_par(
             np.array([[float(par[0]), float(par[0]) ** 2.0, float(par[1])]]).T, moments
         )
         start = [0.0, 1.0]
-        bounds = [(-1.0, 1.0), (0.0, None)]
+        bounds = [(-0.99, 0.99), (0.0, None)]
     if moments[0].shape[1] == 4:
         optim_par = lambda par: foptim_par(
             np.array(
@@ -383,8 +389,13 @@ def optim_moments(moments_in, vcX=np.array([0]), all_par=False, start=None):
         )
         if not start:
             start = [0.0, 1.0, 1.0]
-        bounds = [(-1.0, 1.0), (0.0, None), (0.0, None)]
+        bounds = [(-0.99, 0.99), (0.0, None), (0.0, None)]
     lambdaX = op.fmin_l_bfgs_b(optim_par, start, approx_grad=True, bounds=bounds)
+
+    if hard_bound:
+        if abs(lambdaX[0][0]) >= 0.99:
+            raise Exception("Spatial parameter was outside the bounds of -0.99 and 0.99")
+
     if all_par:
         return lambdaX[0]
     return lambdaX[0][0]
@@ -503,7 +514,7 @@ def get_lags_split(w, x, max_lags, split_at):
     max_lags  : integer
               Maximum order of spatial lag
     split_at: integer
-              Separates the resulting lags into two groups: up to split_at and above
+              Separates the resulting lags into two cc: up to split_at and above
 
     Returns
     --------
@@ -621,7 +632,7 @@ def inverse_prod(
 def power_expansion(
     w, data, scalar, post_multiply=False, threshold=0.0000000001, max_iterations=None
 ):
-    """
+    r"""
     Compute the inverse of a matrix using the power expansion (Leontief
     expansion).  General form is:
 
@@ -662,7 +673,7 @@ def power_expansion(
     return running_total
 
 
-def set_endog(y, x, w, yend, q, w_lags, lag_q, slx_lags=0):
+def set_endog(y, x, w, yend, q, w_lags, lag_q, slx_lags=0,slx_vars="All"):
     # Create spatial lag of y
     yl = lag_spatial(w, y)
     # spatial and non-spatial instruments
@@ -689,15 +700,17 @@ def set_endog(y, x, w, yend, q, w_lags, lag_q, slx_lags=0):
         raise Exception("invalid value passed to yend")
     if slx_lags == 0:
         return yend, q
-    else:
-        return yend, q, lag_x
+    else:  # ajdust returned lag_x here using slx_vars
+        if (isinstance(slx_vars,list)):     # slx_vars has True,False
+            if len(slx_vars) != x.shape[1] :
+                raise Exception("slx_vars incompatible with x column dimensions")
+            else:  # use slx_vars to extract proper columns
+                vv = slx_vars * slx_lags
+                lag_x = lag_x[:,vv]
+            return yend, q, lag_x
+        else:  # slx_vars is "All"
+            return yend, q, lag_x
 
-    lag = lag_spatial(w, x)
-    spat_lags = lag
-    for i in range(w_lags - 1):
-        lag = lag_spatial(w, lag)
-        spat_lags = sphstack(spat_lags, lag)
-    return spat_lags
 
 
 def set_endog_sparse(y, x, w, yend, q, w_lags, lag_q):
@@ -734,7 +747,7 @@ def iter_msg(iteration, max_iter):
     return iter_stop
 
 
-def sp_att(w, y, predy, w_y, rho):
+def sp_att(w, y, predy, w_y, rho, hard_bound=False):
     xb = predy - rho * w_y
     if np.abs(rho) < 1:
         predy_sp = inverse_prod(w, xb, rho)
@@ -742,11 +755,16 @@ def sp_att(w, y, predy, w_y, rho):
         # Note 1: Here if omitting pseudo-R2; If not, see Note 2.
         resid_sp = y - predy_sp
     else:
-        # warn = "Warning: Estimate for rho is outside the boundary (-1, 1). Computation of true inverse of W was required (slow)."
-        # predy_sp = inverse_prod(w, xb, rho, inv_method="true_inv")
-        warn = "*** WARNING: Estimate for spatial lag coefficient is outside the boundary (-1, 1). ***"
-        predy_sp = np.zeros(y.shape, float)
-        resid_sp = np.zeros(y.shape, float)
+        if hard_bound:
+            raise Exception(
+                "Spatial autoregressive parameter is outside the maximum/minimum bounds."
+            )
+        else:
+            # warn = "Warning: Estimate for rho is outside the boundary (-1, 1). Computation of true inverse of W was required (slow)."
+            # predy_sp = inverse_prod(w, xb, rho, inv_method="true_inv")
+            warn = "*** WARNING: Estimate for spatial lag coefficient is outside the boundary (-1, 1). ***"
+            predy_sp = np.zeros(y.shape, float)
+            resid_sp = np.zeros(y.shape, float)
     # resid_sp = y - predy_sp #Note 2: Here if computing true inverse; If not,
     # see Note 1.
     return predy_sp, resid_sp, warn
@@ -789,6 +807,142 @@ def RegressionProps_basic(
     if vm is not None:
         reg.vm = vm
 
+def optim_k(trace, window_size=None):
+    """
+    Finds optimal number of regimes for the endogenous spatial regimes model
+    using a method adapted from Mojena (1977)'s Rule Two.
+
+    Parameters
+    ----------
+    trace      : list
+                    List of SSR values for different number of regimes
+    window_size : integer
+                    Size of the window to be used in the moving average
+                    (Defaults to N//4)
+    Returns
+    -------
+    i+window_size : integer
+                    Optimal number of regimes
+
+    Examples
+    --------
+    >>> import libpysal as ps
+    >>> import numpy as np
+    >>> import spreg
+    >>> data = ps.io.open(ps.examples.get_path('NAT.dbf'))
+    >>> y = np.array(data.by_col('HR90')).reshape((-1,1))
+    >>> x_var = ['PS90','UE90']
+    >>> x = np.array([data.by_col(name) for name in x_var]).T
+    >>> w = ps.weights.Queen.from_shapefile(ps.examples.get_path("NAT.shp"))
+    >>> x_std = (x - np.mean(x,axis=0)) / np.std(x,axis=0)
+    >>> reg = spreg.Skater_reg().fit(20, w, x_std, {'reg':spreg.OLS,'y':y,'x':x}, quorum=100, trace=True)
+    >>> spreg.optim_k([reg._trace[i][1][2] for i in range(1, len(reg._trace))])
+    9
+
+
+    """
+
+    N = len(trace)
+    if not window_size:
+        window_size = N//4 # Mojena suggests from 70% to 90%
+        if window_size < 2:
+            window_size = N
+    std_dev = [np.std(trace[i:i+window_size]) for i in range(N - window_size + 1)]
+    ma = np.convolve(trace, np.ones(window_size)/window_size, mode='valid')
+    treshold = [True]
+    i = 0
+    while treshold[-1] and i < (N - window_size):
+        b = (6/(window_size*(window_size*window_size-1))
+            )*((2*np.sum(np.arange(1, i+2)*trace[window_size-1:i+window_size])
+            )-((window_size+1)*np.sum(trace[window_size-1:i+window_size])))
+        l = (window_size-1)*b/2
+        treshold.append(trace[i+window_size] < ma[i] - b - l - 2.75*std_dev[i])
+        i += 1
+    return i+window_size
+
+
+def make_wnslx(coords,params,leafsize=30,distance_metric='Euclidean'):
+    '''
+    
+    Computes transformed distances as triangular kernel weights for transform = 'power', or fraction of maximum distance 
+    (bandwidth) for transform = 'exponential'. Uses libpysal.cg.KDTree. The three main characteristics of the kernel weights
+    are passed as a tuple with k (number of nearest neighbors), upper_distance_bound (variable or fixed bandwidth),
+    and transform (power or exponential).
+
+    With distance_upper_bound=np.inf, the computation is for a variable bandwidth, where the bandwidth 
+    is determined by the number of nearest neighbors, k. When a distance_upper_bound is set that is larger than the 
+    largest k-nearest neighbor distance, there is no effect. When the distance_upper_bound is less than the max 
+    k-nearest neighbor distance for a given point, then it has the effect of imposing a fixed bandwidth, and 
+    truncating the number of nearest neighbors to those within the bandwidth. As a result, the number of neighbors 
+    will be less than k.
+
+    Note that k is a binding constraint, so if imposing the bandwidth is important, k should be taken large enough.
+
+    Parameters
+    ----------
+    coords               : n by 2 numpy array of x,y coordinates
+    params               : tuple with
+       k                 : number of nearest neighbors (the diagonal is not included in k, so to obtain
+                           k real nearest neighbors KDTree query must be called with k+1
+       distance_upper_bound : bandwidth (see above for interpretation), np.inf is for no bandwidth, used by
+                           KDTree query
+       transform         : determines type of transformation, triangular kernel weights for 'power',
+                           fractional distance for 'exponential' (exponential)
+    leafsize             : argument to construct KDTree, default is 30 (from sklearn)
+    distance_metric      : type of distance, default is "Euclidean", other option is "Arc" for arc-distance, to be used with long,lat
+                           (note: long should be x and lat is y), both are supported by libpysal.cg.KDTree, but not
+                           by its scipy and sklearn counterparts
+
+
+    Returns
+    -------
+    spdis                : transformed distance matrix as CSR sparse array
+
+    '''
+    k = params[0]
+    distance_upper_bound = params[1]
+    transform = params[2]
+    kdt = KDTree(coords,leafsize=leafsize,distance_metric=distance_metric)
+    dis,nbrs = kdt.query(coords,k=k+1,distance_upper_bound=distance_upper_bound) 
+    # get rid of diagonals
+    dis = dis[:,1:]
+    nbrs = nbrs[:,1:]
+    n = dis.shape[0]
+
+    # maximum distance in each row
+    if (np.isinf(distance_upper_bound)): # no fixed bandwidth
+        mxrow = dis[:,-1].reshape(-1,1)
+    else:
+        dis = np.nan_to_num(dis,copy=True,posinf=0)   # turn inf to zero
+        mxrow = np.amax(dis,axis=1).reshape(-1,1)
+
+    # rescaled distance
+    fdis = dis / mxrow
+
+    if transform.lower() == 'power':   # triangular kernel weights
+        fdis = -fdis + 1.0
+    elif transform.lower() == 'exponential':   # distance fraction
+        fdis = fdis
+    else:
+        raise Exception("Method not supported")
+
+    # turn into COO sparse format and then into CSR
+    kk = fdis.shape[1]
+    rowids = np.repeat(np.arange(n),kk)
+    if (np.isinf(distance_upper_bound)):
+        colids = nbrs.flatten()
+        fdis = fdis.flatten()
+    else: # neighbors outside bandwidth have ID n
+        pickgd = (nbrs != n)
+        rowids = rowids[pickgd.flatten()]
+        colids = nbrs[pickgd].flatten()
+        fdis = fdis[pickgd].flatten()
+    
+    spdis = coo_array((fdis,(rowids,colids)))
+    spdis = spdis.tocsr(copy=True)
+    
+    return spdis
+
 
 def _test():
     import doctest
